body	{
	color:	#333333;
	padding-right: 40px;
	padding-left: 25px;
	font-family: Verdana, Arial, Helvetica, sans-serif;
	padding-bottom: 20px;
	padding-top: 40px;
}

h1	{
	font-size: 24px;
}

a.image	{
	text-decoration: none;
	color:#FFFFFF;
}

a	{
	text-decoration: none;
	color: #000099;
	font-weight: bold;
}