BEGIN TRANSACTION;
CREATE TABLE userinfo (key varchar(255), value varchar(255));
CREATE TABLE module (modulename varchar(255), ordering integer primary key);
INSERT INTO "module" VALUES('Thout::Library::SQLLibrary', 1);
INSERT INTO "module" VALUES('Thout::Book::ThoutBook_1_0_jar', 2);
CREATE TABLE book_handler (type varchar(255), routine varchar(255));
INSERT INTO "book_handler" VALUES('Thout_1_0_jar', 'Thout::Book::ThoutBook_1_0_jar');
INSERT INTO "book_handler" VALUES('Thout_1_0', 'Thout::Book::ThoutBook_1_0');
CREATE TABLE library (type varchar(255), name varchar(255), ordering integer primary key);
INSERT INTO "library" VALUES('SQLLibrary', 'SQL_Library.db', 1);
CREATE TABLE library_handler (type varchar(255), routine varchar(255));
INSERT INTO "library_handler" VALUES('SQLLibrary', 'Thout::Library::SQLLibrary');
CREATE TABLE filters (type varchar(255), name varchar(255), ordering integer primary key);
CREATE TABLE crypto (generic_data varchar(255), ordering integer primary key);
COMMIT;
