div.sect3    {
	padding-left: 20pt;
}

h3    {
	background-color: rgb(156, 156, 97);
	color: rgb(255, 255, 255);
}

div.sect2    {
	padding-left: 10pt;
}

h2    {
	background-color: rgb(129, 129, 61);
	color: rgb(255, 255, 255);
}

body    {
	font-family: Arial,Helvetica,sans-serif;
	font-size: 12px;
	color: #000000;
	background-color: #FFFFFF;
	margin-left: 0px;
	margin-top: 0px;
	margin-right: 0px;
	margin-bottom: 0px;
}

h1    {
	font-family: Arial,Helvetica,sans-serif;
	font-size: 14px;
	font-weight: bold;
	color: #FFFFFF;
	display: block;
	background-color: #666600;
	text-indent: 5pt;
	margin: 0px;
	margin-bottom: 2px;
	padding: 0px;
	border-width: 1px;
	border-style: solid;
	border-color: #663300;
}

div.sect1    {
	text-indent: 10pt;
	display: block;
	margin-left: 5px;
	margin-top: 2px;
}

ol    {
	text-indent: 0pt;
}

ul    {
	text-indent: 0pt;
}

li    {
	padding: 3px;
}

pre    {
	diplay: block;
	margin: 2px;
	margin-top: 5px;
	margin-bottom: 5px;
	background-color: #eeeeee;
	padding: 2px;
	color: #000066;
	border-width: 1px;
	border-style: solid;
	border-color: #999999;
	font-size: 9px;
}

table    {
	font-size: 10px;
	border-width: 1px;
	border-style: solid;
	border-color: #999999;
	background-color: #eeeeee;
	margin: 4px;
	margin-left: 0px;
	padding: 0px;
	text-indent: 0px;
}

td    {
	border-width: 1px;
	border-style: solid;
	border-color: #999999;
	background-color: #FFFFFF;
	padding: 2px;
	margin: 1px;
	margin-right: 2px;
	text-indent: 0px;
}

table.orlist    {
	border-width: 0px;
	border-style: none;
	background-color: #ffffff;
	margin: 0px;
	margin-top: 10px;
	margin-left: 10px;
	padding: 0px;
	text-indent: 0px;
}

table.orlist td    {
	border-width: 0px;
	border-style: none;
	background-color: #ffffff;
	margin: 0px;
	margin-left: 10px;
	padding: 0px;
	text-indent: 0px;
}

table.figure    {
	border-width: 0px;
	border-style: none;
	background-color: #ffffff;
	margin: 0px;
	margin-top: 3px;
	margin-left: 10px;
	padding: 0px;
}

table.figure td    {
	font-size: 9px;
	border-width: 0px;
	border-style: none;
	background-color: #ffffff;
	margin: 0px;
	margin-left: 10px;
	padding: 0px;
	text-indent: 0px;
}

*.listing    {
	background-color: #ffffff;
	border: 0px;
	width: 100%;
	border-style: none;
	border-color: #ffffff;
}

*.listing_body    {
	background-color: #ffffff;
	border-color: #ffffff;
	border: 0px;
	width: 100%;
	border-style: none;
}

*.listing_head a    {
	color: #FFFFFF;
}

*.listing_footer a    {
	color: #FFFFFF;
	text-decoration: none;
}

*.figure_footer a    {
	color: #000000;
}

*.listing_head    {
	background-color: #999999;
	color: #FFFFFF;
	border-width: 1px;
	border-bottom-width: 3px;
	border-style: solid;
	border-color: #666666;
	font-size: 12pt;
}

*.listing_footer    {
	background-color: #999999;
	border-width: 1px;
	border-style: solid;
	border-color: #666666;
	text-align: right;
	font-size: 8px;
	padding: 0px;
}

p.note    {
	background-color: #FFFFCC;
	border-color: #999999;
	border-style: solid;
	border-width: 1px;
	margin: 4px;
	padding: 3px;
}

p.hint    {
	background-color: #EAEBFF;
	border-color: #000099;
	border-style: solid;
	border-width: 1px;
	margin: 4px;
	padding: 3px;
}

img    {
	border-color: rgb(0, 0, 0);
	border-style: solid;
	border-width: thin;
}

